#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <bi-config.h>
#include <bi-common.h>
#include <bbxml.h>

#include "babinfoxcom.h"

extern int getesconfig(int efn, struct stefrc *, char *);
extern int setesconfig(int efn, struct stefrc *);
extern int reloadesdrv(int efn);
extern struct stdaqinfo daqinfo;
extern struct struninfo runinfo;

int babinfoxcom(char *buff, char *ret){
  BBXMLEL *dom, *top, *tnode, *rdom, *rtop;
  int sz=0, i;

  rdom = bbxml_DomDocument();
  dom = bbxml_parsebuff(buff, strlen(buff));
  rtop = bbxml_appendChild(rdom, bbxml_createElement("babinfo", ""));
			   
  if((top = bbxml_node(dom, "babinfoxcom"))){
    i = 0;
    while(xcom[i].name){
      tnode = bbxml_node(top, xcom[i].name);
      if(tnode){
	(*xcom[i].func)(tnode, rtop);
      }
      i++;
    }
  }else{
    bbxml_appendChild(rtop, bbxml_createElement("error", "babinfoxcom: This is babinfo(XML), but commond is not babinfoxcom"));
  }
  
  bbxml_free(dom);
  sz = bbxml_sprintall(rdom, ret);
  bbxml_free(rdom);

  return sz;
}


/* babinfo functions */
int xgetesconfig(BBXMLEL *nd, BBXMLEL *ret){
  char tstr[BBXML_TEXTMAX], myhost[256];
  int efn;
  BBXMLEL *efnel, *esconfig, *efar;
  struct stefrc efrc;

  efnel = bbxml_node(nd, "efn");
  esconfig = bbxml_appendChild(ret, bbxml_createElement("esconfig", ""));
  while(efnel){
    efar = bbxml_appendChild(esconfig, bbxml_createElement("node", ""));
    bbxml_appendChild(efar, bbxml_createElement("efn", efnel->text));
    efn = strtol(efnel->text, NULL, 0);
    memset(myhost, 0, sizeof(myhost));
    if(getesconfig(efn, &efrc, myhost)){
      DB(printf("xcom: efn=%d, erhost=%s, myhost=%s\n", efn, efrc.erhost, myhost));
      bbxml_appendChild(efar, bbxml_createElement("myhost", myhost));
      bbxml_appendChild(efar, bbxml_createElement("host", efrc.erhost));
      sprintf(tstr, "%d", efrc.mt);
      bbxml_appendChild(efar, bbxml_createElement("isdrv", tstr));
      bbxml_appendChild(efar, bbxml_createElement("rtdrv", efrc.mtdir));
      sprintf(tstr, "%d", efrc.connect);
      bbxml_appendChild(efar, bbxml_createElement("connect", tstr));
    }else{
      sprintf(tstr, "xgetesconfig: connection error for %s(efn=%d)", myhost, efn);
      bbxml_appendChild(efar, bbxml_createElement("error", tstr));
    }
    efnel = bbxml_next(efnel, "efn");
  }

  return 1;
}


int xsetesconfig(BBXMLEL *nd, BBXMLEL *ret){
  char tstr[BBXML_TEXTMAX], myhost[256];
  int efn;
  BBXMLEL *efnel, *host, *rtdrv;
  struct stefrc efrc;

  if(runinfo.runstat){
    bbxml_appendChild(ret, bbxml_createElement("error", "setesconfig: Now running!"));
    return 0;
  }

  efnel = bbxml_node(nd, "efn");
  if(efnel){
    efn = strtol(efnel->text, NULL, 0);
    memset(myhost, 0, sizeof(myhost));
    if(getesconfig(efn, &efrc, myhost)){
      host = bbxml_node(nd, "host");
      rtdrv = bbxml_node(nd, "rtdrv");
      if(host) strcpy(efrc.erhost, host->text);
      if(rtdrv) strcpy(efrc.mtdir, rtdrv->text);
      setesconfig(efn, &efrc);
    }else{
      sprintf(tstr, "xsetesconfig: connection error for %s(efn=%d)", myhost, efn);
      bbxml_appendChild(ret, bbxml_createElement("error", tstr));
    }
  }else{
    sprintf(tstr, "xsetesconfig: Invalid command (no efn)");
    bbxml_appendChild(ret, bbxml_createElement("error", tstr));
  }

  return 1;
}

int xreloadesdrv(BBXMLEL *nd, BBXMLEL *ret){
  char tstr[BBXML_TEXTMAX];
  int efn;
  BBXMLEL *efnel;
  
  if(runinfo.runstat){
    bbxml_appendChild(ret, bbxml_createElement("error", "reloadesdrv: Now running!"));
    return 0;
  }

  efnel = bbxml_node(nd, "efn");
  if(efnel){
    efn = strtol(efnel->text, NULL, 0);
    if(efn > 0 && efn < 256){
      reloadesdrv(efn);
    }else{
      sprintf(tstr, "xreloadesdrv: Invalid efn %d", efn);
      bbxml_appendChild(ret, bbxml_createElement("error", tstr));
    }
  }else{
    sprintf(tstr, "xreloadesdrv: Invalid command (no efn)");
    bbxml_appendChild(ret, bbxml_createElement("error", tstr));
  }
  
  return 1;
}

